import requests
from lxml import objectify
from lxml.etree import ElementBase
import ast
import json


class NWSCAPParser:
    def __init__(self, raw_cap_xml=None, cap_url=None):
        self.xml = raw_cap_xml
        self.url = cap_url

        if self.xml is None and self.url is None:
            raise Exception('You must pass one parameter "raw_cap_xml" or "cap_url"')

        # placeholders
        self.alert = None
        self.FIPS6 = None
        self.UGC = None
        self.INFO_PARAMS = None

        # init the obj
        self.load()

    def load(self):
        if self.xml is None:
            if self.url:
                self.xml = requests.get(self.url).text
        # print('xml=', self.xml)
        self.alert = objectify.fromstring(self.xml.encode())
        self.FIPS6 = [g.value for g in self.alert.info.area.geocode if str(g.valueName).upper() == 'FIPS6']
        self.FIPS6 = []
        self.UGC = [g.value for g in self.alert.info.area.geocode if str(g.valueName).upper() == 'UGC']
        self.INFO_PARAMS = {}
        [self.INFO_PARAMS.update({p.valueName: p.value}) for p in self.alert.info.parameter]

    def __getattr__(self, name):
        if name.startswith('__'):
            raise AttributeError(name)
        return getattr(self.alert, name)

    def as_dict(self):
        # return ast.literal_eval(repr(self.alert))
        # return objectify.dump(self.alert)
        # return vars(self.alert)
        ret = DumpNode(self.alert)
        return ret

    def as_json(self):
        return json.dumps(self.as_dict(), indent=2)

    def __repr__(self):
        return '<NWSCAPParser.NWSCAPParser instance (identifier:%s)>' % self.identifier


def DumpNode(node):
    d = {}
    for key, value in vars(node).items():
        if isinstance(value, ElementBase):
            value = DumpNode(value)
        d[key] = value
    if len(d) is 0:
        return str(node)
    return d



import sys
from pprint import pprint

from nwscapparser3 import NWSCAPParser


def test_basic_fields(alert):
    print('---- basic fields ----')
    print(alert.identifier)
    print(alert.info.effective)
    print(f'{len(alert.FIPS6)} FIPS6 codes:{alert.FIPS6}')
    print(f'{len(alert.UGC)} UGC codes:{alert.UGC}')
    print(f'{len(alert.INFO_PARAMS)} INFO_PARAMS:{alert.INFO_PARAMS}')


def test_dict_dump(alert):
    print('---- dict dump ----')
    pprint(alert.as_dict())


def test_json_dump(alert):
    print('---- json dump ----')
    pprint(alert.as_json())


if __name__ == '__main__':
    # first command line arg is assumed to be a full URL to a CAP
    import requests

    if len(sys.argv) > 1:
        cap_url = sys.argv[1]
        response = requests.get(cap_url)
        src = response.text
    # testing
    else:
        filepath = r'cap.IL124CA04A2F50.SevereThunderstormWarning.xml'
        with open(filepath, 'rt') as file:
            src = file.read()

    alert = NWSCAPParser(src)
    print(alert)
    test_basic_fields(alert)
    test_dict_dump(alert)
    test_json_dump(alert)
